// Credits :
// NB: net clock based on simplestesp8266clock.ino

// Library to be added manually :
// (Navigate to Sketch > Include Library > Add .ZIP Library. At the top of the drop down list, select the option to "Add .ZIP Library''. You will be prompted to select the library you would like to add)
// Time.h & TimeLib.h:  https://github.com/PaulStoffregen/Time
#include <ESP8266WiFi.h>
#include <WifiUDP.h>
#include <String.h>
#include <NTPClient.h>
#include <Time.h>
#include <TimeLib.h>
#include <Timezone.h>
#include <Adafruit_NeoPixel.h>

// Debug mode
#define DEBG

// Define your wifi password
const char* ssid = "xxx";
const char* password = "xxxx";

// Define NTP properties
#define NTP_OFFSET   60 * 60      // In seconds
#define NTP_INTERVAL 60 * 1000    // In miliseconds
#define NTP_ADDRESS  "fr.pool.ntp.org"  // change this to whatever pool is closest (see ntp.org)

// Set up the NTP UDP client
WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, NTP_ADDRESS, NTP_OFFSET, NTP_INTERVAL);

// Which pin on the Arduino is connected to the NeoPixels?
#define PIN        2

// Words table
const String wordt[] = {
  "il","**",
  "est","****",
  "deux","******",
  "trois","********",
  "quatre","**********",
  "neuf","******",
  "une","****",
  "sept","******",
  "cinq","******", 
  "six","****",
  "huit","******",
  "mi","**",
  "di","**",
  "x","*",
  "minuit","*********",
  "s","*",
  "heure","********",
  "onze","******",
  "moins","********",
  "le","**",
  "dix","****",
  "quart","********",
  "et","**",
  "vingt","********",
  "cinq2","******",
  "demi", "******",
  "Z",""
};

// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS 136 
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);
#define DELAYVAL 60000 // Time (in milliseconds) between time calls Must be 60000

// Init clock
void iniclock(){
#ifdef DEBG
    Serial.begin(115200);
#endif
 pixels.begin(); // INITIALIZE NeoPixel strip object (REQUIRED)
 pixels.clear(); // Set all pixel colors to 'off'
 light("il",255,0,0);
 light("est",0,255,0);
 light("minuit",0,0,255);
 pixels.show();
 timeClient.begin();   // Start the NTP UDP client
 WiFi.begin(ssid, password);
 while (WiFi.status() != WL_CONNECTED) 
  {
    delay(500);
#ifdef DEBG
    Serial.print(".");
#endif
  }
#ifdef DEBG
 Serial.println("Connected to WiFi");
#endif
}

// Get net time
time_t gettime(){
 if (WiFi.status() == WL_CONNECTED) //Check WiFi connection status
  {   
    // update the NTP client and get the UNIX UTC timestamp 
    timeClient.update();
    unsigned long epochTime =  timeClient.getEpochTime();
    // convert received time stamp to time_t object
    time_t local, utc;
    utc = epochTime;
    // Then convert the UTC UNIX timestamp to local time for Paris timezone
    TimeChangeRule usEDT = {"EDT", Last, Sun, Mar, 2, +60};  // UTC + 1 hours - change this as needed
    TimeChangeRule usEST = {"EST", Last, Sun, Oct, 2, +0};   //UTC + 1 hour - change this as needed
    Timezone France(usEDT, usEST);
    local = France.toLocal(utc)%(24*60*60);
    return(local);
      }
  else // attempt to connect to wifi again if disconnected
  {
    Serial.println("Connecting to Wifi...");
    WiFi.begin(ssid, password);
    return(0);
  }
}

// Lights word what with color r,g,b
void light(String what,int r,int g,int b)
{
  #ifdef DEBG
  Serial.println(what);
  #endif
  int i=0,ids=0,ide=0;
  while (wordt[i] != what){
    if (wordt[i][0] == 'Z') {
      return ;
      }
     ids+=wordt[i+1].length();
     i+=2;
    }
  for (int j=0;j<wordt[i+1].length();++j)
    pixels.setPixelColor(ids+j, pixels.Color(r,g,b));
}

// Displays time hour mins
void showtime(int hours,int mins,int r,int g,int b){
 int m=(mins%60)/5;
 int h=(hours+(m>=7?1:0))%24;
 pixels.clear(); // Set all pixel colors to 'off'
 light("il",r,g,b);
 light("est",r,g,b);
 
  // Hours
 if (h>12)
   h=h-12;
  switch (h) {
    case 0:
      light("minuit",r,g,b);
      break;
    case 1:
      light("une",r,g,b);
      light("heure",r,g,b);
      break;
    case 2:
      light("deux",r,g,b);
      light("heure",r,g,b);
      light("s",r,g,b);
      break;
    case 3:
      light("trois",r,g,b);
      light("heure",r,g,b);
      light("s",r,g,b);
      break;
    case 4:
      light("quatre",r,g,b);
      light("heure",r,g,b);
      light("s",r,g,b);
      break;
    case 5:
      light("cinq",r,g,b);
      light("heure",r,g,b);
      light("s",r,g,b);
      break;
    case 6:
      light("six",r,g,b);
      light("heure",r,g,b);
      light("s",r,g,b);
      break;
    case 7:
      light("sept",r,g,b);
      light("heure",r,g,b);
      light("s",r,g,b);
      break;
    case 8:
      light("huit",r,g,b);
      light("heure",r,g,b);
      light("s",r,g,b);
      break;
    case 9:
      light("neuf",r,g,b);
      light("heure",r,g,b);
      light("s",r,g,b);
      break;
    case 10:
      light("di",r,g,b);
      light("x",r,g,b);
      light("heure",r,g,b);
      light("s",r,g,b);
      break;
    case 11:
      light("onze",r,g,b);
      light("heure",r,g,b);
      light("s",r,g,b);
      break;
    case 12:
      light("mi",r,g,b);
      light("di",r,g,b);
      break;}

  switch (m){
    case 0:
      break;
    case 1:
      light("cinq2",r,g,b);
      break;
    case 2:
      light("dix",r,g,b);
      break;
    case 3:
      light("et",r,g,b);
      light("quart",r,g,b);
      break;
    case 4:
      light("vingt",r,g,b);
      break;
    case 5:
      light("vingt",r,g,b);
      light("cinq2",r,g,b);
      break;
    case 6:
      light("et",r,g,b);
      light("demi",r,g,b);
      break;
    case 7:
      light("moins",r,g,b);
      light("vingt",r,g,b);
      light("cinq2",r,g,b);
      break;
    case 8:
      light("moins",r,g,b);
      light("vingt",r,g,b);
      break;
    case 9:
      light("moins",r,g,b);
      light("le",r,g,b);
      light("quart",r,g,b);
      break;
    case 10:
      light("moins",r,g,b);
      light("dix",r,g,b);
      break;
    case 11:
      light("moins",r,g,b);
      light("cinq2",r,g,b);
      break;}
    pixels.show();   // Send the updated pixel colors to the hardware.
}

void setup() {
  iniclock();
#ifdef DEBG
  Serial.println("Init done");
#endif
}

void loop() {
  time_t local=(gettime())%(24*60*60);
  int hours=local/(60*60);
  int mins=(local/60)%60;
  int remd=1+(mins%5); // Reminder
  showtime(hours,mins,255*(remd&1),255*((remd&2)>>1),255*((remd&4)>>1));
  delay(DELAYVAL); // Pause before next pass through loop
}